// -------------------------------------------------------------------------------------------------
// Modul:     GSM
// Funkce:    Obsluha GSM modemu, prijem a cteni SMS, prozvaneni
// Pozadavky: HW UART a jeden GPIO pro indikacni LED 
// -------------------------------------------------------------------------------------------------
         

// bitove flagy stavu USARTu
#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<PE)
#define DATA_OVERRUN (1<<DOR)
#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC) 

// ruzne
#define FALSE 0
#define TRUE  1

// zdroj pro vypis retezcu
#define MEMORY 1
#define FLASH  0

// stavova LED pripojeni modemu
#define ONLINELED_OFF    PORTD |= (1<<PD7);
#define ONLINELED_ON     PORTD &= ~(1<<PD7);
#define ONLINELED_IOSET  DDRD |= (1<<PD7); 

// rele na pripojeni napajeni do GSM modemu
#define MODEMPOWER_ON    PORTB |= (1<<PB3);
#define MODEMPOWER_OFF   PORTB &= ~(1<<PB3); 
#define MODEMPOWER_IOSET DDRB |= (1<<PB3); 

// komunikacni buffer pro UART
#define RX_BUFFER_SIZE 40
volatile char unsigned rx_buffer_pos = 0;
volatile unsigned char rx_buffer[RX_BUFFER_SIZE];   

// sktruktura pro ulozeni SMS zprav
#define TEL_SIZE   13
#define TEXT_SIZE  20
typedef struct {
  unsigned char tel[TEL_SIZE+1];
  unsigned char text[TEXT_SIZE+1];
} structMessage;

// fronta zprav
#define TXFRONTA_MAX_COUNT 5
volatile structMessage TXFronta[TXFRONTA_MAX_COUNT];
volatile unsigned char TXFrontaData = FALSE;          
volatile unsigned char TXFrontaIndex = 0;

// promenna na ulozeni prijate zpravy a docasne id zpravy
volatile structMessage PrijataSMS;
volatile char gsmId[3] = {0,0,0};

// pocet chybnych pokusu o detekci modemu
#define PWRRESTART 10
unsigned char gsmFailedInits = 0; 
            
// seznam stavu komunikace s modemem
#define NOT_COMM               0
#define MODEM_INIT            17
#define MODEM_INIT2           18
#define CHECK_PRESENT         20
#define SMS_REQUEST           21
#define SMS_READ              22
#define SMS_READ2             23
#define SMS_READ3             24
#define SMS_READ4             25
#define SMS_READ5             26
#define SMS_SEND              30
#define SMS_SEND1             31
#define SMS_SEND2             32
#define GSM_RINGRING          33

// retezec OK pro testy odpovedi od modemu
char GSM_AT_OK[] PROGMEM  = "OK";

// makro pro vypnuti timeoutu komunikace
#define COMMTIMEOUT_OFF       0

// perioda spousteni komunikace v sec
#define COMM_PERIOD           10

// priznak, zda-li pripojeny GSM modem 
volatile unsigned char gsmIsModemInitialized = FALSE; 

// casovy limit pro komunikaci s modemem
volatile unsigned int gsmCommTimeout = 0;

// casovac pro periodicke spousteni komunikace
volatile unsigned int gsmCommTimer = COMM_PERIOD;

// stav komunikace se modemem
volatile unsigned int gsmState = NOT_COMM;

// priznak nove prichozi SMS
volatile unsigned int gsmNewSMS = FALSE;


// prototypy
static void gsmSendChar(char c);
static void gsmSendString(char *sss, unsigned char zdroj);
static void gsmStartComm();
static unsigned char gsmIsInBufferFromLeft(char *sss, char zdroj);
void gsmHeartBeat(void);
void gsmRing(char *tel, char zdrojtel);
unsigned char gsmSendSMS(char *tel, char zdrojtel, char *text, char zdrojtext);
void gsmInit(void);


// -------------------------------------------------------------------------------------------------
// [PUBLIC] Funkce na vlozeni cisla a textu fronty SMS zprav k odeslani
// Vstupy   *tel        - ukazatel na string
//           zdrojtel   - 1=string v RAM, 0=string ve flash
//          *text       - ukazatel na string
//           zdrojtext  - 1=string v RAM, 0=string ve flash
// -------------------------------------------------------------------------------------------------
unsigned char gsmSendSMS(char *tel, char zdrojtel, char *text, char zdrojtext) {
  char znak;
  char i, index, found = FALSE;
  // nalezeni volne pozice pro zapis nove SMS do fronty
  for(index=0; index<TXFRONTA_MAX_COUNT; index++) {
    if( (TXFronta[index].tel[0]==0) && (TXFronta[index].text[0]==0) ) {
      found = TRUE;
      break;
    }
  }
  // pokud nebylo zadne volne misto nalezeno
  if(!found) return(FALSE);
  // vlozeni do fronty na volnou pozici
  i=0;
  while ( i<TEL_SIZE ) {
    if (zdrojtel) znak=(*tel++); else znak=pgm_read_byte(tel++);
    if (!znak) break;
    TXFronta[index].tel[i++] = znak;
    TXFronta[index].tel[i] = 0;
  }
  i=0;
  while ( i<TEXT_SIZE ) {
    if (zdrojtext) znak=(*text++); else znak=pgm_read_byte(text++);
    if (!znak) break;
    TXFronta[index].text[i++] = znak;
    TXFronta[index].text[i] = 0;
  }
  TXFrontaData = TRUE;
  if( gsmState==NOT_COMM ) gsmCommTimer=2;
  return(TRUE);
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PRIVATE] Funkce na porovnani textu retezce v RAM s obsahem bufferu zleva
// Vstup:  *sss - retezec v RAM
// Vystup:  TRUE  - retezec sss je obsazen v bufferu 
//          FALSE - retezec neni obsazen v bufferu 
// -------------------------------------------------------------------------------------------------
static unsigned char gsmIsInBufferFromLeft(char *sss, char zdroj) {
  char k,i=0;
  if (rx_buffer[0]==0) return(FALSE); 
  if(zdroj) {
    while ((k=(*sss++))>0) if ( rx_buffer[i++] != k ) return(FALSE);
  } else {
    while ((k=pgm_read_byte(sss++))>0) if ( rx_buffer[i++] != k ) return(FALSE);
  }
  return(TRUE);
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PUBLIC] Funkce je volana casovacem (1s) a slouzi k realizaci timoutu pro komunikaci
// -------------------------------------------------------------------------------------------------
void gsmHeartBeat(void) {
  
  // dekrementace casovace pro spusteni komunikace s modemem
  // jde o to, aby se komunikace poustela periodicky po nekolika sekundach
  // na nule se zastavi
  if( gsmCommTimer==1 ) gsmStartComm();
  if( gsmCommTimer ) gsmCommTimer--;
  
  // casovac komunikace s modemem - realizuje timeout cteni
  // na nule se vypne
  // pruchodem 1 proveden akce
  if( gsmCommTimeout == 1 ) {
    debugSendString(PSTR("[GSM] Modem not respond\n"), FLASH);
    gsmIsModemInitialized = FALSE;
    ONLINELED_OFF;
    gsmState = NOT_COMM;
    gsmCommTimer = COMM_PERIOD;
    gsmFailedInits++;
    if(gsmFailedInits==PWRRESTART) { 
      MODEMPOWER_OFF; 
      gsmFailedInits = 0;
      debugSendString(PSTR("[GSM] Power restart\n"), FLASH);
    } 
  }
  if( gsmCommTimeout ) gsmCommTimeout--;
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PRIVATE] Posle znak na UART 
// Vstup:   c - znak k odeslani na UART
// -------------------------------------------------------------------------------------------------
static void gsmSendChar(char c) {
  while (!(UCSRA & DATA_REGISTER_EMPTY));
  UDR=c;
}  
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PRIVATE] Posle retezec na UART
// Vstup:   *sss - retezec v RAM nebo FLASH
// -------------------------------------------------------------------------------------------------
static void gsmSendString(char *sss, unsigned char zdroj){
  char k;       
  if(zdroj) {
    while ((k=(*sss++))>0) gsmSendChar(k);
  } else {
    while ((k=pgm_read_byte(sss++))>0) gsmSendChar(k);
  }
} 
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [INT] Obsluha preruseni pri prichodu znaku na UART (od GSM modemu)
// -------------------------------------------------------------------------------------------------
ISR(USART_RXC_vect) {
  unsigned char status = UCSRA, data = UDR, n, pu, k, mts;
  
  // test korektnosti prijateho znaku
  if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))!=0) return;

  // postupne naplnovani bufferu s ochranou proti overflow
  if( ( data >= 32 ) && ( rx_buffer_pos < RX_BUFFER_SIZE ) ) {
    rx_buffer[rx_buffer_pos++] = data;
  }
    
  // reakce na znak konce radku nebo zobacek pri odesilani SMS
  if( (data=='\n') || ((data=='>') && (gsmState==SMS_SEND2)) ) {  
    
    // rozdeleni dle typu prave provadene komunikace
    switch (gsmState){
    
      // --- inicializace modemu ---
    
      // reakce na odpoved zmeny na texmode, pozadavek na nastaveni pameti
      case MODEM_INIT:
        if( gsmIsInBufferFromLeft(GSM_AT_OK, FLASH) ) {
          debugSendString(PSTR("[GSM] Set textmode "), FLASH);
          gsmSendString(PSTR("AT+CPMS=\"MT\",\"MT\",\"MT\"\r"), FLASH);
          gsmState = MODEM_INIT2;
          gsmCommTimeout = 5;
        } 
        break;
      
      // reakce na nastaveni pameti. Pokud OK, tak je modem zivy a korekne nastaven
      case MODEM_INIT2:
        if( gsmIsInBufferFromLeft(GSM_AT_OK, FLASH) ) {
          debugSendString(PSTR("& memory \n"), FLASH);
          gsmCommTimeout = COMMTIMEOUT_OFF;
          gsmState = NOT_COMM;
          gsmIsModemInitialized = TRUE;
          ONLINELED_ON;
          gsmCommTimer = COMM_PERIOD;
        } 
        break;
                
      // --- cteni a smazani sms ---

      // reakce na pozadavek cteni SMS, vyparsovani ID a ODESILATELE    
      case SMS_READ:
        if( gsmIsInBufferFromLeft(PSTR("+CMGL:"), FLASH) ) {
          // ulozeni ID zpravy 
          gsmId[0] = rx_buffer[7];
          if( rx_buffer[8] != ',' ) gsmId[1] = rx_buffer[8]; else gsmId[1]=0; 
          // ulozeni telefonniho cisla odesilatele
          n=0;
          pu=0;
          k=0;
          while(n++<rx_buffer_pos) {
            if ( rx_buffer[n]=='"' ) pu++;
            if ( (pu==3) && (rx_buffer[n]>='0') && (rx_buffer[n]<='9') ) {
              PrijataSMS.tel[k] = rx_buffer[n];
              PrijataSMS.tel[++k]=0;
            }
          }  
          gsmState=SMS_READ2;
        }
        // zadna SMS
        if( gsmIsInBufferFromLeft(GSM_AT_OK, FLASH) ) {
          gsmCommTimeout = COMMTIMEOUT_OFF;
          gsmState = NOT_COMM;      
          gsmCommTimer = COMM_PERIOD;   
        }
        break;
      
      // cteni textu prvni prijate zpravy
      case SMS_READ2:
        n=0;
        while( n < rx_buffer_pos ) {
          PrijataSMS.text[n++] = rx_buffer[n];       // prepis zpravy do bufferu
          PrijataSMS.text[n] = 0;                    // doplneni znaku konce retezce
          if (n == TEXT_SIZE) break;           // orez, pokud je zprava delsi jak buffer zpravy
        }
        gsmState=SMS_READ3;
        break;

      // cekani na OK po seznamu zprav a nasledne smazani nactene zpravy
      case SMS_READ3:
        if( gsmIsInBufferFromLeft(GSM_AT_OK, FLASH) ) {     
          gsmSendString(PSTR("AT+CMGD="), FLASH);
          gsmSendString(gsmId, MEMORY);
          gsmSendChar('\r');
          gsmState = SMS_READ4;
          gsmCommTimeout = 5;
        }   
        break;
        
      // potvrzeni o smazani zpravy
      case SMS_READ4:
        if( gsmIsInBufferFromLeft(GSM_AT_OK, FLASH) ) {  
          debugSendString(PSTR("[GSM] <- "), FLASH);
          debugSendString(gsmId, MEMORY);
          debugSendChar(':');
          debugSendString(PrijataSMS.tel, MEMORY);
          debugSendChar(':');
          debugSendString(PrijataSMS.text, MEMORY);
          debugSendChar('\n');
          gsmCommTimeout = COMMTIMEOUT_OFF;
          gsmState = NOT_COMM;   
          gsmNewSMS = TRUE;
          gsmCommTimer = COMM_PERIOD;
        }   
        break;
        
      // --- odeslani fronty SMS ---
     
      // zahajeni odesilani SMS zpravy 
      case SMS_SEND:
        if( gsmIsInBufferFromLeft(GSM_AT_OK, FLASH) ) {
          TXFrontaIndex=0;
          mts=FALSE; 
          while (TXFrontaIndex < TXFRONTA_MAX_COUNT) {
            if( TXFronta[TXFrontaIndex].tel[0] && TXFronta[TXFrontaIndex].text[0] ) {
               mts=TRUE;
               break;
            } else {
              TXFrontaIndex++;
            }
          }
          if(mts) {
            debugSendString(PSTR("[GSM] -> "), FLASH);
            debugSendString(TXFronta[TXFrontaIndex].tel, MEMORY);
            debugSendChar(':');
            gsmSendString(PSTR("AT+CMGS=\"+"), FLASH);
            gsmSendString(TXFronta[TXFrontaIndex].tel, MEMORY);
            gsmSendString(PSTR("\"\r"), FLASH);
            gsmState = SMS_SEND2;
            gsmCommTimeout = 8;
          } else {
            gsmCommTimeout = COMMTIMEOUT_OFF;
            gsmState = NOT_COMM;
            TXFrontaData = FALSE;  
            gsmCommTimer = COMM_PERIOD;         
          }
        }
        break;
        
      // odesilani SMS zprav, text  
      case SMS_SEND2:
        if( gsmIsInBufferFromLeft(PSTR(">"), FLASH) ) {
          gsmSendString(TXFronta[TXFrontaIndex].text, MEMORY);
          debugSendString(TXFronta[TXFrontaIndex].text, MEMORY);
          debugSendChar('\n');
          gsmSendChar(0x1a);
          // smazani zaznamu ve fronte zprav
          TXFronta[TXFrontaIndex].tel[0]  = 0;
          TXFronta[TXFrontaIndex].text[0] = 0;
          gsmState = SMS_SEND;
          gsmCommTimeout = 8;        
        }
        break;   
    
    
    }
      
    // reset vstupniho bufferu
    rx_buffer_pos=0;
    rx_buffer[0]=0;
  }

}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PRIVATE] Vstupni bod do stavoveho prostoru komunikace s GSM modemem
// -------------------------------------------------------------------------------------------------
static void gsmStartComm() {

  // dle pripravenosti modemu
  switch (gsmIsModemInitialized) {
  
    // modem neni pripraven
    case FALSE:
    
      // pokusime se ho detekovat a nastavit
      if( gsmState==NOT_COMM ) {
        MODEMPOWER_ON;
        debugSendString(PSTR("[GSM] Modem init\n"), FLASH);
        gsmSendString(PSTR("AT+CMGF=1\r"), FLASH);
        gsmState = MODEM_INIT;
        gsmCommTimeout = 5; // cas 5 sekund na odpoved
      }
      
      break;
      
    // modem je pripraven  
    case TRUE:
    
      // odesleme frontu SMS zprav
      if( ( gsmState==NOT_COMM ) && ( TXFrontaData ) ) {
        debugSendString(PSTR("[GSM] Sending queue of SMS\n"), FLASH);
        gsmSendString(PSTR("AT\r"), FLASH);
        gsmState = SMS_SEND;
        gsmCommTimeout = 5; // cas 5 sekund na odpoved        
      }
    
      // precteme prichozi SMS zpravy, pokud se nema nic odeslat a neni prijata nezpracovana zprava
      if( ( gsmState==NOT_COMM ) && ( !TXFrontaData ) && ( !gsmNewSMS ) ) {
        debugSendString(PSTR("[GSM] Checking new SMS\n"), FLASH);
        gsmSendString(PSTR("AT+CMGL=\"ALL\"\r"), FLASH);
        gsmState = SMS_READ;
        gsmCommTimeout = 15; // cas 8 sekund na odpoved
      }     
      
      break;
  
  }

}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PUBLIC] Prozvoneni
// -------------------------------------------------------------------------------------------------
void gsmRing(char *tel, char zdrojtel) {
  unsigned long x = 0x0FFFFFF;
  
   // modem musi byt nastaveny a bez komunikace 
  if(!gsmIsModemInitialized) return;
  while(gsmState != NOT_COMM);
   // sestaveni hovoru
  debugSendString(PSTR("[GSM] Ringing to "), FLASH);
  debugSendString(tel, zdrojtel);
  debugSendString(PSTR("\r"), FLASH);
  gsmState = GSM_RINGRING;
  gsmSendString(PSTR("ATD+"), FLASH);
  gsmSendString(tel, zdrojtel);
  gsmSendString(PSTR(";\r"), FLASH);
   // casova prodleva  
  while(x--) asm("nop");
   // polozeni hovoru
  gsmSendString(PSTR("ATH\r"), FLASH);
  debugSendString(PSTR("[GSM] Ringing terminated\r"), FLASH);
  gsmState = NOT_COMM;
  gsmCommTimer = COMM_PERIOD;
}
// -------------------------------------------------------------------------------------------------


// -------------------------------------------------------------------------------------------------
// [PUBLIC] Inicializace
// -------------------------------------------------------------------------------------------------
void gsmInit(void) {
  // nastaveni indikacni LED
  ONLINELED_IOSET;
  MODEMPOWER_IOSET;
  ONLINELED_OFF;
  MODEMPOWER_ON;
  // UART  RX: on, TX: on, 9600 baud, 8bit, 1 stop bit, no parity
  UCSRA=0x00;
  UCSRB=0x98; 
  UCSRC=0x86;
  UBRRL=51;
  UBRRH=0x00;
  // povoleni preruseni
  sei();
}
// -------------------------------------------------------------------------------------------------
