/***************************************************************************************************
 Project : Alarm
 Date    : 10.7.2011
 Author  : Jan Matuska, walda@starhill.org                         
 Company : www.starhill.org
 Funkce  : ovladaci logika alarmu
 ---------------------------------------------------------------------------------------------------
 Chip type                : ATMega8
 Clock frequency          : 8.0 MHz (internal RC)
 Memory model             : Small
 Internal SRAM size       : 8 kB 
 EEPROM memory            : 512 B   
 Compiler                 : avr-gcc 4.1.2 (WinAVR 20070525rc2) 
 Fuses and Lock Bits      : Int.RC 8MHz, Start-up time 6CK + 64ms
                            Brown-out detection na 4.0V
                            No locks
***************************************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#define F_CPU 8000000UL  
#include <util/delay.h>
#include "gsm.c"
#include "debug.c"
#include "rfid.c"


// Makro na mazani WDT
#define CLRWDT  asm("wdr")

// Snadnejsi popis EEPROM sekce
#define EEPROM_SECTION  __attribute__ ((section (".eeprom")))

// Alarmove smycky 
#define SMYCKA0 (PINC&(1<<PC2))
#define SMYCKA1 (PIND&(1<<PD2))
#define SMYCKA2 (PINC&(1<<PC3))
#define SMYCKA3 (PINC&(1<<PC4))
#define SMYCKA4 (PINC&(1<<PC5))

// Vstup 230, vstup ext1, tlacitko
#define IN230  (!(PINB&(1<<PB2)))
#define INEXT  (PINC&(1<<PC0))
#define INKEY  (!(PINB&(1<<PB0)))

// Telefonni cisla
char CISLO_WALDA[] PROGMEM  = "420608337726";
char CISLO_AJA[]   PROGMEM  = "420737578545";

// Stavy alarmu
#define ALARM_TO_DEACTIVATE     0
#define ALARM_DEACTIVATED       1
#define ALARM_TO_ACTIVATE       2
#define ALARM_ACTIVATED         3
#define ALARM_CHECK_INPUTS      4
#define ALARM_TO_SIRENA         5
#define ALARM_SIRENA            6
#define ALARM_TIME_TO_ACTIVATE  7
#define ALARM_DELAY_ALERT       8
#define ALARM_ALERT             9
#define ALARM_FINISH_ALERT      10
#define ALARM_LAST_STATE        10

// Indikacni LED a beeper na panelu RFID ctecky
#define LED_ALARM_ON    PORTB&=~(1<<PB5)
#define LED_ALARM_OFF   PORTB|=(1<<PB5)
#define LED_ALARM_STATE !(PORTB&(1<<PB5))
#define BEEPER_OFF      PORTB&=~(1<<PB4)
#define BEEPER_ON       PORTB|=(1<<PB4)
#define BEEPER_STATE    PORTB&(1<<PB4)
#define BEEP            BEEPER_ON; _delay_ms(100); BEEPER_OFF; _delay_ms(100); 

// Vystupy
#define SIRENA_ON   PORTB|=(1<<PB7)
#define SIRENA_OFF  PORTB&=~(1<<PB7)
#define EXT1_ON     PORTB|=(1<<PB1)
#define EXT1_OFF    PORTB&=~(1<<PB1)
#define EXT2_ON     PORTD|=(1<<PD6)
#define EXT2_OFF    PORTD&=~(1<<PD6)
#define SW230_ON    PORTD|=(1<<PD5)
#define SW230_OFF   PORTD&=~(1<<PD5)

// Pocet cidel a struktura citacu cidel
#define PSENSORS 10
typedef struct {
  unsigned int backref;            // stav senzoru historicky
  unsigned int sensors;            // stavajici stav sensoru
  unsigned int tta[PSENSORS];      // kroku do aktivace senzoru
  unsigned int tto[PSENSORS];      // kroku do dalsi mozne aktivace po odezneni signalu
  unsigned int change[PSENSORS];   // zmeny stavu TNO, TOA, TOD
} structSensors;
structSensors s;

// Zmenove stavy cidel
#define TNO 0
#define TOH 1
#define TOL 2

// Stav alarmu a zalohovany stav
unsigned char alarm = 0;
uint8_t ee_alarm  EEPROM_SECTION = 1;

// Globalni pole pro praci s retezcem
unsigned char mystring[31];

// Delicka pro blikani LED ctecky a pipani
volatile unsigned char counter;
volatile unsigned char divider;

// Ruzne
#define FALSE 0
#define TRUE  1
#define BOOL char

#define NOEOL 0
#define EOL   1

#define READER_FASTBEEP   TCCR0=0x05; divider=0x03;
#define READER_SLOWBEEP   TCCR0=0x05; divider=0x0F; 
#define READER_SILENT     TCCR0=0x00; 

// stavy 
#define EI_ODPOJIT    0
#define EI_PRIPOJIT   1
#define EI_MODE_VYP   0
#define EI_MODE_ZAP   1
#define EI_MODE_PERM  2
#define EI_MODE_IMP   3
#define EI_LAST_MODE  3
unsigned char eimode = EI_MODE_VYP;
uint8_t ee_eimode  EEPROM_SECTION = EI_MODE_VYP;



// Prototypy
unsigned char SearchString(char *sss, char zdroj, char *kde);
void CopyToTempBuffer(char *sss, char zdroj, char bufpos, char eol);
void SMSProcess(void);
unsigned char ReadVoltage(void);
void ReadSensors(void);


// -------------------------------------------------------------------------------------------------
// sekundove tiky od casovace Timer 1
// -------------------------------------------------------------------------------------------------
ISR(TIMER1_COMPA_vect) {
  // takt pro provoz GSM modemu
  gsmHeartBeat();
  // smazne psa
  CLRWDT;
  // takt pro provoz rfid
  rfidHeartBeat;
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// blikani LED + pipani na ctecce
// -------------------------------------------------------------------------------------------------
ISR(TIMER0_OVF_vect) {
  counter++;
  if(!(counter&divider)) {
    if(LED_ALARM_STATE) {LED_ALARM_OFF;} else {LED_ALARM_ON; BEEPER_ON;}
  }
  if(!(~counter&0x03)) BEEPER_OFF;
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// Funkce na porovnani 2 retezcu 
// Vstup:  *sss   - hledany retezec
//          zdroj - umisteno hledaneho retezce
//          kde   - prohledavany retezec v RAM 
// Vystup:  TRUE  - retezec sss je obsazen v kde
//          FALSE - retezec neni obsazen v kde 
// -------------------------------------------------------------------------------------------------
unsigned char SearchString(char *sss, char zdroj, char *kde) {
  char k,i=0;
  if (kde[0]==0) return(FALSE); 
  if(zdroj) {
    while ((k=(*sss++))>0) if ( kde[i++] != k ) return(FALSE);
  } else {
    while ((k=pgm_read_byte(sss++))>0) if ( kde[i++] != k ) return(FALSE);
  }
  return(TRUE);
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// Funkce kopii retezce do pomocneho pole mystring
// Vstup:   *sss   - vstupni retezec
//          zdroj  - zdroj retezce FLASH nebo MEMORY
//          bufpos - pozice v mystring, od ktere se bude *sss zapisovat
//          eol    - TRUE/FALSE, pokud se ma nakonec *sss zapsat znak konce retezce 0x0  
// -------------------------------------------------------------------------------------------------
void CopyToTempBuffer(char *sss, char zdroj, char bufpos, char eol) {
  char k,i=0;
  if(zdroj) {
    while ((k=(*sss++))>0) mystring[(i++)+bufpos]=k; 
  } else {
    while ((k=pgm_read_byte(sss++))>0) {mystring[(i++)+bufpos]=k;}
  }
  if(eol) mystring[i+bufpos]=0; 
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// Zpracovani SMS zprav
// -------------------------------------------------------------------------------------------------
void SMSProcess(void) {

   unsigned char i,j,k,retval;

   // Zpracovani SMS zpravy pro zapnuti alarmu
   if( SearchString(PSTR("Azap"), FLASH, PrijataSMS.text) ) {
     alarm = ALARM_TO_ACTIVATE;
     gsmSendSMS(PrijataSMS.tel, MEMORY,  PSTR("Alarm aktivovan!"), FLASH);
   }   
   
   // Zpracovani SMS zpravy pro vypnuti alarmu
   if( SearchString(PSTR("Avyp"), FLASH, PrijataSMS.text) ) {
     alarm = ALARM_TO_DEACTIVATE;
     gsmSendSMS(PrijataSMS.tel, MEMORY,  PSTR("Alarm deaktivovan!"), FLASH);
   }   
   
   // Zapnuti rele externich vystupu 
   // "Arelezap x", kde x=1..2 je vystup
   if( SearchString(PSTR("Arelezap"), FLASH, PrijataSMS.text) ) {
     i = PrijataSMS.text[9]-'0';
     if((i>=1) && (i<=2)) {
       CopyToTempBuffer(PSTR("Rele - zapnuto"), FLASH, 0, EOL);
       mystring[5] = PrijataSMS.text[9];
       gsmSendSMS(PrijataSMS.tel, MEMORY,  mystring, MEMORY);
       switch(i) {
         case 1:  EXT1_ON; break;
         case 2:  EXT2_ON; break;
       }
     }
   }
   
   // Vypnuti rele externich vystupu 
   // "Arelevyp x", kde x=1..2 je vystup
   if( SearchString(PSTR("Arelevyp"), FLASH, PrijataSMS.text) ) {
     i = PrijataSMS.text[9]-'0';
     if((i>=1) && (i<=2)) {
       CopyToTempBuffer(PSTR("Rele - vypnuto"), FLASH, 0, EOL);
       mystring[5] = PrijataSMS.text[9];
       gsmSendSMS(PrijataSMS.tel, MEMORY,  mystring, MEMORY);
       switch(i) {
         case 1:  EXT1_OFF; break;
         case 2:  EXT2_OFF; break;
       }
     }
   }   

   // Nastaveni rezimu odpojovani elektroinstalace
   // Aeimode x, kde x=0..3 dle konstant EI_MODE_xxx
   if( SearchString(PSTR("Aeimode"), FLASH, PrijataSMS.text) ) {
     i = PrijataSMS.text[8]-'0';
     if(i<=EI_LAST_MODE) {
       CopyToTempBuffer(PSTR("EImode -"), FLASH, 0, EOL);
       mystring[7]=PrijataSMS.text[8];
       gsmSendSMS(PrijataSMS.tel, MEMORY,  mystring, MEMORY);
       eeprom_write_byte(&ee_eimode, i);
     }
   }

   // Zpracovani SMS zpravy zapis nove karty do DB
   if( SearchString(PSTR("Aaddcard"), FLASH, PrijataSMS.text) ) {
     i = PrijataSMS.text[9]-'0';
     if( i<=RFID_DBCARDS ) {
       debugSendString(PSTR("[SMS] Pozadavek na zapis karty "), FLASH);
       debugSendChar(PrijataSMS.text[9]);debugSendChar('\n');
       CopyToTempBuffer(PSTR("Karta - zapis"), FLASH, 0, EOL);
       mystring[6]=PrijataSMS.text[9];
       gsmSendSMS(PrijataSMS.tel, MEMORY,  mystring, MEMORY);
       retval=FALSE;
       j=60;
       while(j--) {
         if( rfidNewCardReceived ) {
           rfidAddCard(i);
           retval=TRUE;
           rfidCleanup();
           break;
         }                                
         _delay_ms(1000);
         CLRWDT;  
       }
       if(retval) {
         gsmSendSMS(PrijataSMS.tel, MEMORY, PSTR("Karta - zapis OK"), FLASH);
       } else {
         gsmSendSMS(PrijataSMS.tel, MEMORY, PSTR("Karta - zapis CHYBA"), FLASH);
       }
     }
   }   

   // Zpracovani SMS zpravy pro smazani karty z DB
   if( SearchString(PSTR("Adelcard"), FLASH, PrijataSMS.text) ) {
     i = PrijataSMS.text[9] - '0';
     if( i<=RFID_DBCARDS ) {
       CopyToTempBuffer(PSTR("Karta - smazana"), FLASH, 0, EOL);
       mystring[6]=PrijataSMS.text[9];
       rfidDeleteCard(i);
       gsmSendSMS(PrijataSMS.tel, MEMORY,  mystring, MEMORY);
     }
   }   
   
   // Zpracovani SMS zpravy pro zaslani stavu vstupu IN230 a INEXT
   if( SearchString(PSTR("Ainstav"), FLASH, PrijataSMS.text) ) {
     if(alarm==ALARM_DEACTIVATED) {
       CopyToTempBuffer(PSTR("IN230: -, INEXT: -"), FLASH, 0, EOL);
       if(s.sensors&(1<<7)) mystring[7]='1'; else mystring[7]='0';
       if(s.sensors&(1<<8)) mystring[17]='1'; else mystring[17]='0'; 
       gsmSendSMS(PrijataSMS.tel, MEMORY,  mystring, MEMORY);
     }
   }    
   
   // Zpracovani SMS zpravy pro zaslani stavu alarmu
   if( SearchString(PSTR("Astav"), FLASH, PrijataSMS.text) ) {
     if(alarm==ALARM_DEACTIVATED) {
       gsmSendSMS(PrijataSMS.tel, MEMORY,  PSTR("Alarm je vypnut."), FLASH);
     } else {
       gsmSendSMS(PrijataSMS.tel, MEMORY,  PSTR("Alarm je zapnut."), FLASH);
     }
   }   

   // Zpracovani SMS zpravy zjisteni napeti zdroje
   if( SearchString(PSTR("Anap"), FLASH, PrijataSMS.text) ) {
     i = ReadVoltage();
     itoa(i, mystring, 10);
     if(i<100) { mystring[2]=mystring[1]; mystring[1]='.'; mystring[3]='V'; mystring[4]=0; }  
     if(i>=100) { mystring[3]=mystring[2]; mystring[2]='.'; mystring[4]='V'; mystring[5]=0; }  
     gsmSendSMS(PrijataSMS.tel, MEMORY,  mystring, MEMORY);
     debugSendString(PSTR("[PWR] Napeti je "), FLASH);
     debugSendString(mystring, MEMORY);
     debugSendString(PSTR("\n"), FLASH);
   } 
    
   // Nastaveni priznaku, ze SMS byla zpracovava
   gsmNewSMS = FALSE;
   
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// Zmereni napajeciho napeti 
// Vystup: napeti ve stovkach mV 
// -------------------------------------------------------------------------------------------------
unsigned char ReadVoltage(void) {
  unsigned int x;
  ADCSRA |= (1<<ADSC);
  while ((ADCSRA & (1<<ADSC)));
  x = (ADCH * 106) / 100;
  return((unsigned char) x);
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// Zjisteni stavu cidel a nastaveni zmenovych stavu
// Vstup:  ukazatel na strukturu structSensors
// -------------------------------------------------------------------------------------------------
void ReadSensors(void) {
  unsigned char i, now, tta_val, tto_val, napeti=ReadVoltage();
  for(i=0; i<PSENSORS; i++) {
    // -- zjisteni stavu --
    tta_val=3; tto_val=3;
    switch (i) {
      case 0:
        now = SMYCKA0; break;
      case 1:
        now = SMYCKA1; break;
      case 2:
        now = SMYCKA2; break;
      case 3:
        now = SMYCKA3; break;
      case 4:
        now = SMYCKA4; break;
      case 5:
        if(s.sensors&(1<<i)) {
          if (napeti>=135) now=0; else now=1;  // nad 13.5V => zasuvka
        } else {
          if (napeti<130) now=1; else now=0;   // pod 13V => baterka
        }
        tta_val=60; tto_val=60;
        break;
      case 6:
        if(s.sensors&(1<<i)) {
          if (napeti>=115) now=0; else now=1;   // nad 11.5V => normalni napeti
        } else {
          if (napeti<110) now=1; else now=0;    // pod 10.5V => kriticke napeti
        }
        tta_val=60; tto_val=60;
        break;
      case 7:
        now = IN230;
        tta_val=60; tto_val=60;
        break;
      case 8:
        now = INEXT;
        tta_val=60; tto_val=60;
        break;
      case 9:
        now = INKEY; 
        tta_val=10; tto_val=10;
        break;
    }   
    // -- detekce eventu --    
    s.change[i] = TNO;                
    if( s.sensors&(1<<i) ) {
      if( now ) {
        s.tto[i]=0;
      } else {
        if( s.tto[i]<0x0FFFF ) s.tto[i]++;
        if( s.tto[i]>=tto_val ) { s.sensors &= ~(1<<i); s.change[i] = TOL; }
      }
    } else {
      if( now ) {
        if( s.tta[i]<0x0FFFF ) s.tta[i]++;
        if( s.tta[i]>=tta_val ) { s.sensors |= (1<<i); s.backref |= (1<<i); s.change[i] = TOH; }
      } else {
        s.tta[i]=0;
      }
    }
  }
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// Nastaveni stavu elektroinstalace
// Vstup:  udaj, zda-li se ma elektroinstalace pripojit nebo odpojit
// -------------------------------------------------------------------------------------------------
void SetEI(unsigned char t) {
  switch(eimode) {
    case EI_MODE_VYP:
      SW230_OFF; 
      break; 
    case EI_MODE_ZAP:
      SW230_ON; 
      break; 
    case EI_MODE_PERM:
      if(t==EI_PRIPOJIT) SW230_ON; 
      if(t==EI_ODPOJIT)  SW230_OFF;
      break;
    case EI_MODE_IMP:
      if( (t==EI_PRIPOJIT) && !IN230 ) { SW230_ON; _delay_ms(1000); SW230_OFF; }
      if( (t==EI_ODPOJIT) && IN230 )   { SW230_ON; _delay_ms(1000); SW230_OFF; }    
      break; 
  }
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// Hlavni funkce
// ------------------------------------------------------------------------------------------------- 
int main(void) {

  unsigned int cnt;
  unsigned char i, event;
  //structSensors sensors;

  _delay_ms(500);

  // nastaveni pullupu smycek, vstupu 230V a ext
  PORTB |= (1<<PB0)|(1<<PB2);
  PORTC |= (1<<PC0)|(1<<PC2)|(1<<PC3)|(1<<PC4)|(1<<PC5);
  PORTD |= (1<<PD2);
 
  // nastaveni vystupnich pinu
  DDRB |=  (1<<PB1)|(1<<PB4)|(1<<PB5);
  DDRD |=  (1<<PD5)|(1<<PD6);
 
  // vychozi stav snimacu - vse OK
  s.sensors=0;
  
  // zapnuti WDT
  CLRWDT;
  WDTCR |= (1<<WDCE) | (1<<WDE);  // spusteni WDT
  WDTCR |= 0x07; 
  
  // nastaveni debugline
  debugInit();

  // nastaveni rfid
  rfidInit();
    
  // nastaveni gsm
  gsmInit();    
    
  // ADC: reference na Vcc, nastaveni vstupu ADC1 a zapnuti ADC
  ADMUX = (1<<REFS0)|(1<<ADLAR)|0x01;
  ADCSRA |= (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
  
  // vychozi stav indikace na panelu rfid ctecky
  BEEPER_OFF;
  LED_ALARM_OFF;
  
  // Nastaveni casovace Timer 1 na sekundove pulsy jako preruseni
  TCCR1B = (1<<CS12) | (1<<CS10) | (1<<WGM12);   // timer1 1024 prescaler, CTC mode
  OCR1AH = 0x1E; 
  OCR1AL = 0x84;
  TIMSK |= (1<<OCIE1A);
  
  // Nastaveni casovace Timer 0 na blikani a pipani ctecky karet
  TCCR0 = 0;
  TIMSK |= (1<<TOIE0);

  // Povoleni preruseni
  sei();
  
  // Vypsani uvitaci zpravy na debugline
  debugSendString(PSTR("Power-on\n"), FLASH);
  
  // Nastaveni puvodniho stavu alarmu
  alarm = eeprom_read_byte(&ee_alarm);
  if(alarm>ALARM_LAST_STATE) {
    alarm = ALARM_DEACTIVATED;
    eeprom_write_byte(&ee_alarm, alarm);
  }
  
  // Nastaveni puvodniho rezimu eimode
  eimode = eeprom_read_byte(&ee_eimode);
  if(eimode>EI_LAST_MODE) {
    eimode = EI_MODE_VYP;
    eeprom_write_byte(&ee_eimode, eimode);
  } 
  SetEI(EI_PRIPOJIT); 


  // -- hlavni smycka --
  while(1) {

    // zpracovani prectenti RFID karty           
    if( rfidNewCardReceived ) {
      if(rfidFindCardInDB()) {
        if(alarm==ALARM_DEACTIVATED) {
          alarm=ALARM_TO_ACTIVATE;
        } else {
          alarm=ALARM_TO_DEACTIVATE;
        }
      }
      rfidCleanup();      
    } 
          
    // zpracovani prichozi SMS zpravy
    if( gsmNewSMS ) {
      SMSProcess();
    }
    
    // Nacteni stavu cidel
    ReadSensors();
      
    // Debug smycek s PIR senzory
    i=5;
    while(i--) {
      event=FALSE;
      if(s.change[i]==TOH) {CopyToTempBuffer(PSTR("[PIR-  ] naruseni\n"), FLASH, 0, EOL); event=TRUE;}
      if(s.change[i]==TOL) {CopyToTempBuffer(PSTR("[PIR-  ] klid\n"), FLASH, 0, EOL); event=TRUE;}
      if(event) {
        if(i==0) CopyToTempBuffer(PSTR("ZA"), FLASH, 5, NOEOL); 
        if(i==1) CopyToTempBuffer(PSTR("OB"), FLASH, 5, NOEOL); 
        if(i==2) CopyToTempBuffer(PSTR("LO"), FLASH, 5, NOEOL); 
        if(i==3) CopyToTempBuffer(PSTR("EL"), FLASH, 5, NOEOL); 
        if(i==4) CopyToTempBuffer(PSTR("KA"), FLASH, 5, NOEOL); 
        debugSendString(mystring, MEMORY);
      }       
    }

    // Hlidani napajeni
    if(s.change[5]==TOH) {
      debugSendString(PSTR("[PWR] Vypadek 230V\n"), FLASH);
      gsmSendSMS(CISLO_WALDA, FLASH, PSTR("Vypadek 230V."), FLASH);
    } 
    if(s.change[5]==TOL) { 
      debugSendString(PSTR("[PWR] Nahozeni 230V\n"), FLASH);
      gsmSendSMS(CISLO_WALDA, FLASH, PSTR("Obnova 230V."), FLASH);
    }
    if(s.change[6]==TOH) {
      debugSendString(PSTR("[PWR] Kriticke podpeti\n"), FLASH);
      gsmSendSMS(CISLO_WALDA, FLASH, PSTR("Kriticke podpeti."), FLASH);
    } 
    if(s.change[6]==TOL) {
      debugSendString(PSTR("[PWR] Napeti v norme\n"), FLASH);
    }
    
    // Napajeni odpojovane vetve elektroinstalace
    if(s.change[7]==TOH) {
      debugSendString(PSTR("[230] Pod napetim\n"), FLASH);
    }
    if(s.change[7]==TOL) {
      debugSendString(PSTR("[230] Bez napeti\n"), FLASH);
    }
    
    // Externi vstup
    if(s.change[8]==TOH) {
      debugSendString(PSTR("[EXT] Detekce\n"), FLASH);
    }
    if(s.change[8]==TOL) {
      debugSendString(PSTR("[EXT] Klid\n"), FLASH);
    }
    
    // Deaktivace / aktivace alarmu tlacitkem na desce 
    if(s.change[9]==TOH) {
      if(alarm==ALARM_DEACTIVATED) {
        alarm=ALARM_TO_ACTIVATE;
      } else {
        alarm=ALARM_TO_DEACTIVATE;
      }  
    }
        
    // -- stavy alarmu --
    switch (alarm) {
      
      // deaktivace alarmu
      case ALARM_TO_DEACTIVATE:
        debugSendString(PSTR("[A] Deaktivace alarmu...\n"), FLASH);
        eeprom_write_byte(&ee_alarm, ALARM_DEACTIVATED);
        READER_SILENT;
        SIRENA_OFF;
        BEEPER_OFF;
        LED_ALARM_OFF;
        _delay_ms(100);
        BEEP; BEEP;  
        SetEI(EI_PRIPOJIT);
        alarm = ALARM_DEACTIVATED;
        break;
      
      // zahajeni aktivace alarmu
      case ALARM_TO_ACTIVATE:
        debugSendString(PSTR("[A] Zahajeni aktivace alarmu...\n"), FLASH);
        eeprom_write_byte(&ee_alarm, ALARM_TO_ACTIVATE);
        cnt = 1200;
        alarm = ALARM_TIME_TO_ACTIVATE;
        READER_SLOWBEEP;
        break;
      
      // odpocitavani do zastrezeni  
      case ALARM_TIME_TO_ACTIVATE:
        if(--cnt==0) {
          debugSendString(PSTR("[A] Alarm aktivovan...\n"), FLASH);
          SetEI(EI_ODPOJIT);
          alarm = ALARM_CHECK_INPUTS;
          READER_SILENT;      
          LED_ALARM_ON;
        }
        break; 
      
      // test zmeny stavu cidel
      case ALARM_CHECK_INPUTS:
        i=5;
        while(i--) {
          if(s.change[i]==TOH) {
            debugSendString(PSTR("[A] Detekce naruseni...\n"), FLASH);
            alarm = ALARM_DELAY_ALERT;
            cnt = 400; 
            READER_FASTBEEP;
          } else {
            s.backref &= ~(1<<i);
          }
        }
        break;
        
      // prodleva pro spusteni poplachu
      case ALARM_DELAY_ALERT:
        if(--cnt==0) {
          debugSendString(PSTR("[A] Spusten poplach...\n"), FLASH);
          alarm = ALARM_ALERT;
        }
        break;
      
      // spusteni poplachu
      case ALARM_ALERT:
        SIRENA_ON;
        LED_ALARM_ON;
        CopyToTempBuffer(PSTR("POPLACH ----------"), FLASH, 0, EOL);
        if(s.backref&(1<<0)) CopyToTempBuffer(PSTR("Za"), FLASH, 8, NOEOL);
        if(s.backref&(1<<1)) CopyToTempBuffer(PSTR("Ob"), FLASH, 10, NOEOL);
        if(s.backref&(1<<2)) CopyToTempBuffer(PSTR("Lo"), FLASH, 12, NOEOL);
        if(s.backref&(1<<3)) CopyToTempBuffer(PSTR("El"), FLASH, 14, NOEOL); 
        if(s.backref&(1<<4)) CopyToTempBuffer(PSTR("Ka"), FLASH, 16, NOEOL);
        gsmSendSMS(CISLO_WALDA, FLASH, mystring, MEMORY);
        cnt=2300;
        alarm=ALARM_FINISH_ALERT;
        break;
      
      // ukonceni poplachu
      case ALARM_FINISH_ALERT:
        if(--cnt==0) {
          debugSendString(PSTR("[A] Ukoncen poplach...\n"), FLASH);
          alarm=ALARM_TO_DEACTIVATE;
        }   
        break;
    }
    // -- /stavy alarmu --
    
    
    // krokovaci cas hlavni smycky
    CLRWDT;
    _delay_ms(50);

  }

}


