// -------------------------------------------------------------------------------------------------
// Modul:     RFID
// Funkce:    Obsluha ctecky RFID karet protokolem weigand 26 a sprava databaze v EEPROM
// Pozadavky: INT0, 2 GPIO pro signly D1 a D0 weigand 26, kdy jeden musi mit INT0
// Spec:      D1 a D0 musi byt premosteno diodou, tak, aby preruseni INTx vyvolal libovolny 
//            prichozi bit.
// -------------------------------------------------------------------------------------------------


// vstupy
#define RFID_D0_IN        (PIND&(1<<PD3))
#define RFID_D1_IN        (PIND&(1<<PD4))
// nastaveni IO pinu + pullup
#define RFID_IO_INPUT     DDRD&=~((1<<PD3)|(1<<PD4)); 
#define RFID_IO_PULLUPS   PORTD|=(1<<PD3)|(1<<PD4);
// timeout pro cteni kodu karty
#define RFID_RX_TIMEOUT   2
// max pocet karet v DB 
#define RFID_DBCARDS      10
// indexy
#define TRUE              1
#define FALSE             0


// Promenne v EEPROM  ------------------------------------------------------------------------------
#define EEPROM_SECTION __attribute__ ((section (".eeprom")))
typedef struct{                                                                                        
  unsigned char   data[4];
} RFIDCARDDB;
RFIDCARDDB ee_rfidCardDB[RFID_DBCARDS] EEPROM_SECTION;
// /Promenne v EEPROM  -----------------------------------------------------------------------------


// globani promenne
volatile unsigned char rfidReceivedData[4];
volatile unsigned char rfidBitCount;
volatile unsigned char rfidTimeout;
volatile unsigned char rfidNewCardReceived;


// prototypy
void rfidInit(void);
void rfidCleanup(void);
void rfidHeartBeat(void);
void rfidDeleteCard(unsigned char index);
void rfidAddCard(unsigned char index);
unsigned char rfidFindCardInDB(void);


// -------------------------------------------------------------------------------------------------
// [INT] Obsluha preruseni na sestupnou hranu INT0
//       Prijem bitu ze ctecky
// -------------------------------------------------------------------------------------------------
ISR (INT1_vect) {
  unsigned char byte_index=rfidBitCount>>3;
  unsigned char bit_index=rfidBitCount&0x07;
  if( RFID_D1_IN ) {
    rfidReceivedData[byte_index]|=(1<<bit_index);
  } else {
    rfidReceivedData[byte_index]&=~(1<<bit_index);
  }
  rfidBitCount++;
  if( rfidBitCount==26 ) {
    rfidReceivedData[3]&=0x03;
    debugSendString(PSTR("[RFID] Prijata nova karta\n"), FLASH);
    rfidNewCardReceived=TRUE;
  }
  rfidTimeout=RFID_RX_TIMEOUT;
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PUBLIC] Provede zakladni nastaveni RFID
// -------------------------------------------------------------------------------------------------
void rfidCleanup(void) {
  rfidBitCount=0;
  rfidNewCardReceived=FALSE;
  rfidTimeout=0;
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PUBLIC] Timeout pro cteni kodu karty - volat 1x za sekundu
// -------------------------------------------------------------------------------------------------
void rfidHeartBeat(void) {
  if( rfidTimeout ) { 
    rfidTimeout--;
    if( (rfidTimeout==0) && (rfidNewCardReceived==FALSE) ) {
      rfidCleanup();
    }  
  }
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PUBLIC] Smaze kod karty z prislusne pozice z DB
//          Vstup: index - pozice kodu karty v DB  
// -------------------------------------------------------------------------------------------------
void rfidDeleteCard(unsigned char index) {
  unsigned char i=4;
  if(index<RFID_DBCARDS) {
    while(i--){
      eeprom_write_byte(&ee_rfidCardDB[index].data[i], 0xAA);
    }
    debugSendString(PSTR("[RFID] Smazani karty "), FLASH);
    debugSendChar(index+'0');debugSendChar('\n');
  }
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PUBLIC] Vlozi kod karty z rfidReceivedData do DB na prislusnou pozici
//          Vstup: index - pozice kodu karty v DB  
// -------------------------------------------------------------------------------------------------
void rfidAddCard(unsigned char index) {
  unsigned char i=4;
  if(index<RFID_DBCARDS) {
    while(i--){
      eeprom_write_byte(&ee_rfidCardDB[index].data[i], rfidReceivedData[i]);
    }
    debugSendString(PSTR("[RFID] Zapis karty "), FLASH);
    debugSendChar(index+'0');debugSendChar('\n');
  }
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PUBLIC] Vyhleda index v DB, kde se nachazi karta nactena v rfidReceivedData
//          Vystup: TRUE/FALSE dle toho, zda-li byl kod karty v DB nalezen nebo ne   
// -------------------------------------------------------------------------------------------------
unsigned char rfidFindCardInDB(void) {
  unsigned char out, i, index=RFID_DBCARDS;
  while(index--) {
    i=4;
    out=TRUE;
    while(i--) {
      if(eeprom_read_byte(&ee_rfidCardDB[index].data[i])!=rfidReceivedData[i]) {
        out=FALSE;
        break;
      } 
    }
    if(out==TRUE) {
      debugSendString(PSTR("[RFID] Karta v DB "), FLASH);
      debugSendChar(index+'0');debugSendChar('\n');
      return(TRUE);
    }
  }
  return(FALSE);
}
// -------------------------------------------------------------------------------------------------



// -------------------------------------------------------------------------------------------------
// [PUBLIC] Provede zakladni nastaveni RFID
// -------------------------------------------------------------------------------------------------
void rfidInit(void) {
  // nastaveni GPIO jako vstup
  RFID_IO_INPUT; 
  // nastaveni pullup na vstupni piny
  RFID_IO_PULLUPS;
  // preruseni na sestupnou hranu INT0
  MCUCR |= (1 << ISC11);
  // povoleni preruseni od INT0 
  GICR |= (1 << INT1);  
  // nastaveni vychoziho stavu
  rfidCleanup();
}
// -------------------------------------------------------------------------------------------------

